"""
Mongo DB API
"""
try:
    from pymongo import MongoClient
except ModuleNotFoundError as e:
    raise ModuleNotFoundError("Please install the library. https://github.com/mongodb/mongo-python-driver")


class MongoDB:
    """Mongo DB table API"""

    def __new__(cls, host: str, port: int, db: str):
        """
        Connect the mongodb database
        """
        client = MongoClient(host, port)
        db_obj = client[db]
        return db_obj


if __name__ == '__main__':
    mongo_db = MongoDB("localhost", 27017, "yapi")
    col = mongo_db.list_collection_names()
    print("collection list: ", col)
    data = mongo_db.project.find_one()
    print("table one data:", data)



"""
lounger CLI
"""
import os
from pathlib import Path

import click
from pytest_req.log import log

from lounger import __version__


@click.command()
@click.version_option(version=__version__, help="Show version.")
@click.option("-pw", "--project-web", help="Create an web automation test project.")
@click.option("-pa", "--project-api", help="Create an api automation test project.")
def main(project_web, project_api):
    """
    lounger CLI.
    """

    if project_web:
        create_scaffold(project_web, "web")
        return 0

    if project_api:
        create_scaffold(project_api, "api")
        return 0


def create_scaffold(project_name: str, type: str) -> None:
    """
    create scaffold with specified project name.
    :param project_name:
    :param type:
    :return:
    """
    if os.path.isdir(project_name):
        log.info(f"Folder {project_name} exists, please specify a new folder name.")
        return

    log.info(f"Start to create new test project: {project_name}")
    log.info(f"CWD: {os.getcwd()}\n")

    def create_folder(path):
        os.makedirs(path)
        log.info(f"created folder: {path}")

    def create_file(path, file_content=""):
        with open(path, 'w', encoding="utf-8") as py_file:
            py_file.write(file_content)
        msg = f"created file: {path}"
        log.info(msg)

    current_file = Path(__file__).resolve()

    # create base file
    conftest_path = current_file.parent / "project_temp" / "conftest.py"
    conftest_content = conftest_path.read_text(encoding='utf-8')
    create_folder(project_name)
    create_folder(os.path.join(project_name, "reports"))

    web_ini = '''[pytest]
log_format = %(asctime)s | %(levelname)-8s | %(filename)s | %(message)s
log_date_format = %Y-%m-%d %H:%M:%S
base_url = https://cn.bing.com
addopts = -s --browser=chromium --headed --html=./reports/result.html
'''
    api_ini = '''[pytest]
log_format = %(asctime)s | %(levelname)-8s | %(filename)s | %(message)s
log_date_format = %Y-%m-%d %H:%M:%S
base_url = https://httpbin.org
addopts = -s --html=./reports/result.html
'''

    if type == "api":
        # create api file
        api_case_path = current_file.parent / "project_temp" / "test_api.py"
        content = api_case_path.read_text(encoding='utf-8')
        create_file(os.path.join(project_name, "test_api.py"), content)
        create_file(os.path.join(project_name, "pytest.ini"), api_ini)
    elif type == "web":
        # create web file
        web_case_path = current_file.parent / "project_temp" / "test_web.py"
        content = web_case_path.read_text(encoding='utf-8')
        create_file(os.path.join(project_name, "test_web.py"), content)
        create_file(os.path.join(project_name, "pytest.ini"), web_ini)


if __name__ == '__main__':
    main()



"""
Data type conversion of different files
"""
import csv
import json
from itertools import islice
import codecs

import yaml
from openpyxl import load_workbook


def check_data(list_data: list) -> list:
    """
    Checking test data format.
    :param list_data:
    :return:
    """
    if isinstance(list_data, list) is False:
        raise TypeError("The data format is not `list`.")
    if len(list_data) == 0:
        raise ValueError("The data format cannot be `[]`.")
    if isinstance(list_data[0], dict):
        test_data = []
        for data in list_data:
            line = []
            for d in data.values():
                line.append(d)
            test_data.append(line)
        return test_data

    return list_data


def csv_to_list(file: str = None, line: int = 1, end_line: int = None) -> list:
    """
    Convert CSV file data to list
    :param file: Path to file
    :param line: Start line of read data
    :param end_line: End line of read data
    :return: list data

    Usage:
        csv_to_list("data.csv", line=1)
    """
    if file is None:
        raise FileExistsError("Please specify the CSV file to convert.")

    table_data = []
    with codecs.open(file, 'r', encoding='utf_8_sig') as csv_file:
        csv_data = csv.reader(csv_file)
        for i in islice(csv_data, line - 1, end_line):
            table_data.append(i)

    return table_data


def excel_to_list(file: str = None, sheet: str = "Sheet1", line: int = 1, end_line: int = None) -> list:
    """
    Convert Excel file data to list
    :param file: Path to file
    :param sheet: Excel sheet, default name is Sheet1
    :param line: Start line of read data
    :param end_line: Start line of read data
    :return: list data

    Usage:
        excel_to_list("data.xlsx", sheet="Sheet1", line=1)
    """
    if file is None:
        raise FileExistsError("Please specify the Excel file to convert.")

    excel_table = load_workbook(file)
    sheet = excel_table[sheet]
    if end_line is None:
        end_line = sheet.max_row

    table_data = []
    for i in sheet.iter_rows(line, end_line):
        line_data = []
        for field in i:
            line_data.append(field.value)
        table_data.append(line_data)

    return table_data


def json_to_list(file: str = None, key: str = None) -> list:
    """
    Convert JSON file data to list
    :param file: Path to file
    :param key: Specifies the key for the dictionary
    :return: list data

    Usage:
        json_to_list("data.yaml", key="login")
    """
    if file is None:
        raise FileExistsError("Please specify the JSON file to convert.")

    if key is None:
        with open(file, "r", encoding="utf-8") as json_file:
            data = json.load(json_file)
            list_data = check_data(data)
    else:
        with open(file, "r", encoding="utf-8") as json_file:
            try:
                data = json.load(json_file)[key]
                list_data = check_data(data)
            except KeyError as exc:
                raise ValueError(f"Check the test data, no '{key}'.") from exc

    return list_data


def yaml_to_list(file: str = None, key: str = None) -> list:
    """
    Convert YAML file data to list
    :param file: Path to file
    :param key: Specifies the key for the dictionary
    :return: list data

    Usage:
        yaml_to_list("data.yaml", key="login")
    """
    if file is None:
        raise FileExistsError("Please specify the YAML file to convert.")

    if key is None:
        with open(file, "r", encoding="utf-8") as yaml_file:
            data = yaml.load(yaml_file, Loader=yaml.FullLoader)
            list_data = check_data(data)
    else:
        with open(file, "r", encoding="utf-8") as yaml_file:
            try:
                data = yaml.load(yaml_file, Loader=yaml.FullLoader)[key]
                list_data = check_data(data)
            except KeyError as exc:
                raise ValueError(f"Check the YAML test data, no '{key}'") from exc

    return list_data



"""
SQLite3 DB API
"""
import sqlite3
from typing import Any

from lounger.db_operation.base_db import SQLBase


class SQLiteDB(SQLBase):
    """SQLite3 DB table API"""

    def __init__(self, db_path: str):
        """
        Connect to the sqlite database
        """
        self.connection = sqlite3.connect(db_path)
        self.cursor = self.connection.cursor()

    def close(self) -> None:
        """
        Close the database connection
        """
        self.connection.close()

    def execute_sql(self, sql: str) -> None:
        """
        Execute SQL
        """
        self.cursor.execute(sql)
        self.connection.commit()

    def insert_data(self, table: str, data: dict) -> None:
        """
        insert sql statement
        """
        for key in data:
            data[key] = "'" + str(data[key]) + "'"
        key = ','.join(data.keys())
        value = ','.join(data.values())
        sql = f"""insert into {table} ({key}) values ({value})"""
        self.execute_sql(sql)

    def query_sql(self, sql: str) -> list:
        """
        Query SQL
        return: query data
        """
        data_list = []
        self.cursor.execute(sql)
        rows = self.cursor.fetchall()
        for row in rows:
            data_list.append(row)
        return data_list

    def query_one(self, sql: str) -> Any:
        """
        Query one data SQL
        return: query data
        """
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        return row

    def insert_get_last_id(self, sql: str) -> int:
        """
        insert sql and get last row id
        :param sql:
        :return: query data
        """
        self.cursor.execute(sql)
        last_id = self.cursor.lastrowid
        self.connection.commit()
        return last_id

    def select_data(self, table: str, where: dict = None, one: bool = False) -> Any:
        """
        select sql statement
        """
        sql = f"""select * from {table} """
        if where is not None:
            sql += f""" where {self.dict_to_str_and(where)}"""
        if one is True:
            return self.query_one(sql)

        return self.query_sql(sql)

    def update_data(self, table: str, data: dict, where: dict) -> None:
        """
        update sql statement
        """
        sql = f"""update {table} set """
        sql += self.dict_to_str(data)
        if where:
            sql += f""" where {self.dict_to_str_and(where)};"""
        self.execute_sql(sql)

    def delete_data(self, table: str, where: dict = None) -> None:
        """
        delete table data
        """
        sql = f"""delete from {table}"""
        if where is not None:
            sql += f""" where {self.dict_to_str_and(where)};"""
        self.execute_sql(sql)

    def init_table(self, table_data: dict, clear: bool = True) -> None:
        """
        init table data
        """
        for table, data_list in table_data.items():
            if clear:
                self.delete_data(table)
            for data in data_list:
                self.insert_data(table, data)



import pytest
import inspect as sys_inspect
import os
from pathlib import Path
from pytest_req.log import log

from lounger.pytest_extend import conversion
from lounger.config import Lounger

__all__ = [
    "file_data", "data",
]


def _search_file_path(file_name: str, file_dir: Path) -> str:
    """
    find file path
    :param file_name:
    :param file_dir:
    """
    file_path = ""
    find_root_dir = file_dir.parent.parent

    for root, _, files in os.walk(find_root_dir, topdown=False):
        for file in files:
            if Lounger.env is not None:
                if root.endswith(Lounger.env) and file == file_name:
                    file_path = os.path.join(root, file_name)
                    break
            else:
                if file == file_name:
                    file_path = os.path.join(root, file_name)
                    break
        else:
            continue
        break

    return file_path


def _search_env_file_path(file_dir: Path, file_part_path: str) -> str:
    """
    find environment file path, Lounger.env != None
    :param file_dir:
    :param file_part_path:
    """
    file_path = ""
    find_root_dir = file_dir.parent
    file_name = file_part_path.split("/")[-1]
    file_part = os.path.join(Lounger.env, file_part_path[:-len(file_name) - 1])

    for root, _, files in os.walk(find_root_dir, topdown=False):
        for file in files:
            if root.endswith(file_part) and file == file_name:
                file_path = os.path.join(root, file_name)
                break
        else:
            continue
        break

    return file_path


def find_file(file: str, file_dir: Path) -> str:
    """
    find file
    :param file:
    :param file_dir:
    """
    if os.path.isfile(file) is True:
        return file

    if "/" in file or "\\" in file:
        file = file.replace("\\", "/")

        if Lounger.env is not None:
            file_path = _search_env_file_path(file_dir=file_dir, file_part_path=file)
            return file_path
        else:
            # Starting at file_dir, search up the 5 levels of parent directories
            for _ in range(5):
                current_dir = os.path.join(file_dir, file)
                if os.path.isfile(current_dir):
                    return current_dir
                file_dir = file_dir.parent  # Move up to the parent directory
            else:
                return ""
    else:
        file_path = _search_file_path(file_dir=file_dir, file_name=file)
        return file_path


def file_data(file: str, line: int = 1, sheet: str = "Sheet1", key: str = None, end_line: int = None):
    """
    Support file parametrize decorator.

    :param file: file name
    :param line: Start line number of an Excel/CSV file
    :param end_line:  End line number of an Excel/CSV file
    :param sheet: Excel sheet name
    :param key: Key name of an YAML/JSON file

    Usage:
    d.json
    ```json
    {
     "login":  [
        ["admin", "admin123"],
        ["guest", "guest123"]
     ]
    }
    ```
    >>  @file_data(file="d.json", key="login")
    ... def test_case(self, username, password):
    ...     print(username)
    ...     print(password)
    """
    if file is None:
        raise FileExistsError("File name does not exist.")

    stack_t = sys_inspect.stack()
    ins = sys_inspect.getframeinfo(stack_t[1][0])
    file_dir = Path(ins.filename).resolve().parent

    if Lounger.env is not None:
        log.info(f"env: '{Lounger.env}', find data file: '{file}'")
    else:
        log.info(f"find data file: {file}")

    file_path = find_file(file, file_dir)
    if file_path == "":
        if Lounger.env is not None:
            raise FileExistsError(f"No '{Lounger.env}/{file}' data file found.")
        raise FileExistsError(f"No '{file}' data file found.")

    suffix = file.split(".")[-1]
    if suffix == "csv":
        data_list = conversion.csv_to_list(file_path, line=line, end_line=end_line)
    elif suffix == "xlsx":
        data_list = conversion.excel_to_list(file_path, sheet=sheet, line=line, end_line=end_line)
    elif suffix == "json":
        data_list = conversion.json_to_list(file_path, key=key)
    elif suffix == "yaml":
        data_list = conversion.yaml_to_list(file_path, key=key)
    else:
        raise FileExistsError(f"Your file is not supported: {file}")

    return data(data_list)


def data(datas, ids: list = None, scope=None):
    """
    lounger parametrize decorator
    :param datas:
    :param ids:
    :param scope:
    :return:
    """
    if ids is None:
        ids = []

    if scope is None:
        scope = "function"

    return pytest.mark.parametrize(
        "params",
        datas,
        ids=ids,
        scope=scope)



import logging
from datetime import datetime, UTC
from io import StringIO
from typing import Any

import pytest
from loguru import logger

from lounger.pytest_extend.screenshot import screenshot_base64

LOG_STREAM = StringIO()

html_title = "Lounger Test Report"


@pytest.hookimpl(tryfirst=True)
def pytest_configure(config):
    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s | %(levelname)-8s | %(filename)s | %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )
    html = config.getoption("--html")
    if html:
        logger.remove()
        logger.add(LOG_STREAM,
                   format="{time: YYYY-MM-DD HH:mm:ss} | <level>{level: <8}</level> | {file: <10} | {message}",
                   level="DEBUG")

    global html_title
    # Here we fetch the command-line argument using config object
    title = config.getoption("--html-title")
    if title:
        html_title = title
    # add env markers
    config.addinivalue_line(
        "markers", "env(name): mark test to run only on named environment"
    )


def pytest_runtest_setup(item: Any) -> None:
    """
    Called to perform the setup phase for a test item.
    """
    env_names = [mark.args[0] for mark in item.iter_markers(name="env")]
    if env_names:
        if item.config.getoption("--env") not in env_names:
            pytest.skip(f"test requires env in {env_names}")


def pytest_html_report_title(report):
    """
    Configures the pytest-html report title based on command-line options.
    :param report:
    :return:
    """
    global html_title
    report.title = html_title


def pytest_html_results_table_header(cells):
    cells.insert(2, "<th>Description</th>")
    cells.insert(3, '<th class="sortable time" data-column-type="time">Time</th>')


def pytest_html_results_table_row(report, cells):
    cells.insert(2, f"<td>{report.description}</td>")
    cells.insert(3, f'<td class="col-time">{datetime.now(tz=UTC).strftime("%Y-%m-%d %H:%M:%S")}</td>')


@pytest.hookimpl(hookwrapper=True)
def pytest_runtest_makereport(item):
    outcome = yield
    pytest_html = item.config.pluginmanager.getplugin('html')
    report = outcome.get_result()
    report.description = str(item.function.__doc__)
    extra = getattr(report, 'extra', [])
    if report.when == 'call':
        xfail = hasattr(report, 'wasxfail')
        if (report.skipped and xfail) or (report.failed and not xfail):
            page = item.funcargs.get('page')
            if page is not None:
                # add screenshot to HTML report.
                image = screenshot_base64(page)
                extra.append(pytest_html.extras.image(image, mime_type='image/png'))

        # add Loguru log to HTML report.
        log_content = LOG_STREAM.getvalue()
        if log_content.strip():
            extra.append(pytest_html.extras.text(log_content, "Loguru Log"))
        # Empty memory stream
        LOG_STREAM.truncate(0)
        LOG_STREAM.seek(0)

    report.extras = extra


def pytest_addoption(parser: Any) -> None:
    """
    Add pytest option
    """
    group = parser.getgroup("lounger", "Lounger")
    group.addoption(
        "--html-title",
        action="store",
        default=[],
        help="Specifies the title of the pytest-html test report",
    )
    group.addoption(
        "--env",
        action="store",
        default=[],
        help="only run tests matching the environment {name}.",
    )



"""
MySQL DB API
"""
from typing import Any

import pymysql.cursors

from lounger.db_operation.base_db import SQLBase


class MySQLDB(SQLBase):
    """MySQL DB table API"""

    def __init__(self, host: str, port: int, user: str, password: str, database: str, charset='utf8mb4'):
        """
        Connect to the MySQL database
        :param host:
        :param port:
        :param user:
        :param password:
        :param database:
        """
        self.connection = pymysql.connect(host=host,
                                          port=int(port),
                                          user=user,
                                          password=password,
                                          database=database,
                                          charset=charset,
                                          cursorclass=pymysql.cursors.DictCursor)

    def close(self) -> None:
        """
        Close the database connection
        """
        self.connection.close()

    def execute_sql(self, sql: str) -> None:
        """
        Execute SQL
        """
        with self.connection.cursor() as cursor:
            self.connection.ping(reconnect=True)
            if "delete" in sql.lower()[0:6]:
                cursor.execute("SET FOREIGN_KEY_CHECKS=0;")
            cursor.execute(sql)
        self.connection.commit()

    def query_sql(self, sql: str) -> list:
        """
        Query SQL
        return: query data
        """
        data_list = []
        with self.connection.cursor() as cursor:
            self.connection.ping(reconnect=True)
            cursor.execute(sql)
            rows = cursor.fetchall()
            for row in rows:
                data_list.append(row)
            self.connection.commit()
            return data_list

    def query_one(self, sql: str) -> Any:
        """
        Query one data SQL
        :return:
        """
        with self.connection.cursor() as cursor:
            self.connection.ping(reconnect=True)
            cursor.execute(sql)
            row = cursor.fetchone()
            self.connection.commit()
            return row

    def insert_get_last_id(self, sql: str) -> int:
        """
        insert sql and get last row id
        :param sql:
        :return:
        """
        with self.connection.cursor() as cursor:
            self.connection.ping(reconnect=True)
            cursor.execute(sql)
            last_id = cursor.lastrowid
            self.connection.commit()
            return last_id

    def insert_data(self, table: str, data: dict) -> None:
        """
        insert sql statement
        """
        for key in data:
            data[key] = "'" + str(data[key]) + "'"
        key = ','.join(data.keys())
        value = ','.join(data.values())
        sql = f"""insert into {table} ({key}) values ({value})"""
        self.execute_sql(sql)

    def select_data(self, table: str, where: dict = None, one: bool = False) -> Any:
        """
        select sql statement
        """
        sql = f"""select * from {table} """
        if where is not None:
            sql += f""" where {self.dict_to_str_and(where)}"""
        if one is True:
            return self.query_one(sql)

        return self.query_sql(sql)

    def update_data(self, table: str, data: dict, where: dict) -> None:
        """
        update sql statement
        """
        sql = f"""update {table} set """
        sql += self.dict_to_str(data)
        if where:
            sql += f""" where {self.dict_to_str_and(where)};"""
        self.execute_sql(sql)

    def delete_data(self, table: str, where: dict = None) -> None:
        """
        delete table data
        """
        sql = f"""delete from {table}"""
        if where is not None:
            sql += f""" where {self.dict_to_str_and(where)};"""
        self.execute_sql(sql)

    def init_table(self, table_data: dict, clear: bool = True) -> None:
        """
        init table data
        """
        for table, data_list in table_data.items():
            if clear:
                self.delete_data(table)
            for data in data_list:
                self.insert_data(table, data)
        self.close()



from datetime import datetime, UTC

import pytest

from lounger.pytest_extend.screenshot import screenshot_base64


def pytest_html_report_title(report):
    report.title = "Lounger Test Report"


def pytest_html_results_table_header(cells):
    cells.insert(2, "<th>Description</th>")
    cells.insert(3, '<th class="sortable time" data-column-type="time">Time</th>')


def pytest_html_results_table_row(report, cells):
    cells.insert(2, f"<td>{report.description}</td>")
    cells.insert(3, f'<td class="col-time">{datetime.now(tz=UTC).strftime("%Y-%m-%d %H:%M:%S")}</td>')


@pytest.hookimpl(hookwrapper=True)
def pytest_runtest_makereport(item):
    outcome = yield
    pytest_html = item.config.pluginmanager.getplugin('html')
    report = outcome.get_result()
    report.description = str(item.function.__doc__)
    extra = getattr(report, 'extra', [])
    if report.when == 'call':
        xfail = hasattr(report, 'wasxfail')
        if (report.skipped and xfail) or (report.failed and not xfail):
            page = item.funcargs.get('page')
            if page is not None:
                # add screenshot to HTML report.
                image = screenshot_base64(page)
                extra.append(pytest_html.extras.image(image, mime_type='image/png'))
        report.extra = extra



"""
playwright test sample

run test：
> pytest --browser=chromium --headed -vs test_web.py
"""
import re
import time

from playwright.sync_api import Page, expect


def test_has_title(page: Page, base_url):
    page.goto(base_url)

    # Expect a title "to contain" a substring.
    expect(page).to_have_title(re.compile("Bing"))


def test_get_started_link(page: Page, base_url):
    page.goto(base_url)
    input = page.locator("#sb_form_q")
    input.type("playwright")
    input.press("Enter")
    time.sleep(5)

    # Expects page to have a heading with the name of "Installation | Playwright".
    search_result = page.get_by_role("link", name='Installation | Playwright').first
    expect(search_result).to_be_visible()



from functools import wraps
from typing import Callable, Text, Tuple

from lounger.log import log
from lounger.utils import cache


def dependent_func(func_obj: Callable, key_name: Text = None, *out_args, **out_kwargs):
    """
    Dependent function decorator.

    :param func_obj: function object.
    :param key_name:
    :param out_args:
    :param out_kwargs:
    :return:
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            func_name = func.__name__
            depend_func_name = func_obj.__name__

            key = key_name
            if key_name is None:
                key = depend_func_name

            if not cache.get(key):
                dependence_res = _call_dependence(func_obj, func_name, *out_args, **out_kwargs)
                cache.set({key: dependence_res})

            else:
                log.info(f"🔗 <{depend_func_name}> a has been executed, obtain it in cache through `{key}`.")
            r = func(*args, **kwargs)
            return r

        return wrapper

    return decorator


def _call_dependence(dependent_api: Callable or Text, func_name: Text, *args, **kwargs) -> Tuple:
    """
    Execution dependent method.
    :param dependent_api:
    :param func_name:
    :param args:
    :param kwargs:
    :return:
    """
    depend_func_name = dependent_api.__name__
    log.info(f"🔗 <{func_name}> depends on <{depend_func_name}>, execute.")
    res = dependent_api(*args, **kwargs)
    return res



from typing import Any

try:
    import psycopg2
    import psycopg2.extras
except ModuleNotFoundError as e:
    raise ModuleNotFoundError("Please install the library. https://github.com/psycopg/psycopg2")
from lounger.db_operation.base_db import SQLBase


class PostgresDB(SQLBase):

    def __init__(self, host: str, port: int, database: str, user: str, password: str):
        """
        Connect to Postgres database
        :param host:
        :param port:
        :param database:
        :param  user:
        :param password:
        """
        self.connection = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
        self.connection.autocommit = True

    def close(self):
        """
        Close the database connection
        """
        self.connection.close()

    def execute_sql(self, sql: str) -> None:
        """
        Execute SQL
        """
        print("runner SQL ", sql)
        with self.connection.cursor() as cursor:
            cursor.execute(sql)

    def query_sql(self, sql: str) -> list:
        """
        Query SQL
        :return: query data

        """
        data_list = []
        with self.connection.cursor(cursor_factory=psycopg2.extras.DictCursor) as cursor:
            cursor.execute(sql)
            rows = cursor.fetchall()
            for row in rows:
                data_list.append(row)
            return data_list

    def query_one(self, sql: str) -> Any:
        """
        Query one row
        """
        with self.connection.cursor(cursor_factory=psycopg2.extras.DictCursor) as cursor:
            cursor.execute(sql)
            row = cursor.fetchone()
            return row

    def insert_get_last_id(self, sql: str) -> int:
        """
        insert sql and get last row id
        :param sql:
        :return:
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            last_id = cursor.lastrowid
            return last_id

    def insert_data(self, table: str, data: dict) -> None:
        """
        insert sql statement
        """
        for key in data:
            data[key] = "'" + str(data[key]) + "'"
        key = ','.join(data.keys())
        value = ','.join(data.values())
        sql = f"INSERT INTO {table} ({key}) VALUES ({value})"
        self.execute_sql(sql)

    def select_data(self, table: str, where: dict = None, one: bool = False) -> Any:
        """
        Select SQL statement
        """
        sql = f"SELECT * FROM {table}"
        if where:
            where_clause = self.dict_to_str_and(where)
            sql += f" WHERE {where_clause}"

        return self.query_one(sql) if one else self.query_sql(sql)

    def update_data(self, table: str, data: dict, where: dict) -> None:
        """
        Update SQL statement
        """
        set_clause = self.dict_to_str(data)
        where_clause = self.dict_to_str_and(where)
        sql = f"UPDATE {table} SET {set_clause} WHERE {where_clause}"
        self.execute_sql(sql)

    def delete_data(self, table: str, where: dict = None) -> None:
        """
        Delete SQL statement
        """
        sql = f"DELETE FROM {table}"
        if where:
            where_clause = self.dict_to_str_and(where)
            sql += f" WHERE {where_clause}"
        self.execute_sql(sql)

    def init_table(self, table_data: dict, clear: bool = True) -> None:
        """
        Initialize table data
        """
        for table, data_list in table_data.items():
            if clear:
                self.delete_data(table)
            for data in data_list:
                self.insert_data(table, data)



import threading


class BrowserConfig:
    """
    Define run browser config
    """
    NAME = None
    REPORT_PATH = None
    REPORT_TITLE = "Lounger Test Report"
    LOG_PATH = None


class Lounger:
    env = None

    _thread_local = threading.local()

    @property
    def action(self):
        """
        playwright locator action
        """
        return getattr(self._thread_local, 'action', None)

    @action.setter
    def action(self, value):
        self._thread_local.action = value



"""
lounger request
"""
import json
from functools import wraps

import requests
from pytest_req.plugin import request
from pytest_req.utils.jmespath import jmespath

from lounger.log import log


class HttpRequest:
    """lounger http request class"""

    def __init__(self, base_url: str = None):
        self.base_url = base_url

    @request
    def get(self, url, params=None, **kwargs):
        if self.base_url is not None:
            url = self.base_url + url
        return requests.get(url, params=params, **kwargs)

    @request
    def post(self, url, data=None, json=None, **kwargs):
        if self.base_url is not None:
            url = self.base_url + url
        return requests.post(url, data=data, json=json, **kwargs)

    @request
    def put(self, url, data=None, **kwargs):
        if self.base_url is not None:
            url = self.base_url + url
        return requests.put(url, data=data, **kwargs)

    @request
    def delete(self, url, **kwargs):
        if self.base_url is not None:
            url = self.base_url + url
        return requests.delete(url, **kwargs)

    @request
    def patch(self, url, data=None, **kwargs):
        if self.base_url is not None:
            url = self.base_url + url
        return requests.patch(url, data=data, **kwargs)


def api(describe: str = "", status_code: int = 200, ret: str = None, check: dict = None, debug: bool = False):
    """
    checkout api response data
    :param describe: interface describe
    :param status_code: http status code
    :param ret: return data
    :param check: check data
    :param debug: debug Ture/False
    :return:
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            func_name = func.__name__
            if debug is True:
                log.debug(f"Execute {func_name} - args: {args}")
                log.debug(f"Execute {func_name} - kwargs: {kwargs}")

            r = func(*args, **kwargs)
            flat = True
            if r.status_code != status_code:
                log.error(f"Execute {func_name} - {describe} failed: {r.status_code}")
                flat = False

            try:
                r.json()
            except json.decoder.JSONDecodeError:
                log.error(f"Execute {func_name} - {describe} failed：Not in JSON format")
                flat = False

            if debug is True:
                log.debug(f"Execute {func_name} - response:\n {r.json()}")

            if flat is True:
                log.info(f"Execute {func_name} - {describe} success!")

            if check is not None:
                for expr, value in check.items():
                    data = jmespath(r.json(), expr)
                    if data != value:
                        log.error(f"Execute {func_name} - check data failed：{expr} = {value}")
                        log.error(f"Execute {func_name} - response：{r.json()}")
                        raise ValueError(f"{data} != {value}")

            if ret is not None:
                data = jmespath(r.json(), ret)
                if data is None:
                    log.error(f"Execute {func_name} - return {ret} is None")
                return data

            return r.json()

        return wrapper

    return decorator



"""
requests test sample

run test：
> pytest -vs test_api.py
"""
from pytest_req.assertions import expect


def test_post_method(post, base_url):
    """
    test post request
    """
    s = post(f'{base_url}/post', data={'key': 'value'})
    expect(s).to_be_ok()


def test_get_method(get, base_url):
    """
    test get request
    """
    payload = {'key1': 'value1', 'key2': 'value2'}
    s = get(f"{base_url}/get", params=payload)
    expect(s).to_be_ok()


def test_session(session, base_url):
    """
    test session, keep requests cookie
    """
    s = session
    s.get(f"{base_url}/cookies/set/sessioncookie/123456789")
    s.get(f"{base_url}/cookies")



"""
seldom cache
"""
import json
import os
import pickle
import shutil
import tempfile
import threading
import uuid
from functools import lru_cache
from functools import wraps as func_wraps

from pytest_req.log import log

DATA_PATH = os.path.join(tempfile.gettempdir(), "cache_data.json")


class Cache:
    """
    Disk Cache through JSON files
    """
    _lock = threading.Lock()  # 创建类级别的锁

    def __init__(self):
        is_exist = os.path.isfile(DATA_PATH)
        if is_exist is False:
            with open(DATA_PATH, "w", encoding="utf-8") as json_file:
                json.dump({}, json_file)

    @staticmethod
    def clear(name: str = None) -> None:
        """
        Clearing cached
        :param name: key
        """
        with Cache._lock:
            if name is None:
                with open(DATA_PATH, "w+", encoding="utf-8") as json_file:
                    json.dump({}, json_file)
                    log.info("💾 Clear all cache data.")
            else:
                with open(DATA_PATH, "r+", encoding="utf-8") as json_file:
                    save_data = json.load(json_file)
                    if name in set(save_data.keys()):
                        del save_data[name]
                        log.info(f"💾 Clear cache data: {name}")

                        json_file.seek(0)
                        json_file.truncate()
                        json.dump(save_data, json_file)

    @staticmethod
    def set(data: dict) -> None:
        """
        Setting cached
        :param data:
        """
        with Cache._lock:
            with open(DATA_PATH, "r+", encoding="utf-8") as json_file:
                save_data = json.load(json_file)
                for key, value in data.items():
                    data = save_data.get(key, None)
                    if data is None:
                        log.info(f"💾 Set cache data: {key} = {value}")
                    else:
                        log.info(f"💾 Update cache data: {key} = {value}")
                    save_data[key] = value

                json_file.seek(0)
                json_file.truncate()
                json.dump(save_data, json_file)

    @staticmethod
    def get(name=None):
        """
        Getting cached
        :param name: key
        :return:
        """
        with Cache._lock:
            with open(DATA_PATH, "r+", encoding="utf-8") as json_file:
                save_data = json.load(json_file)
                if name is None:
                    return save_data

                value = save_data.get(name, None)
                if value is not None:
                    log.info(f"💾 Get cache data: {name} = {value}")

                return value


cache = Cache()


def memory_cache(maxsize=None, typed=False):
    """ memory (Least-recently-used) cache decorator
    """
    return lru_cache(maxsize=maxsize, typed=typed)


class DiskCache:
    """
    Cache data to disk decorator
    """

    _NAMESPACE = uuid.UUID("c875fb30-a8a8-402d-a796-225a6b065cad")

    def __init__(self, cache_path=None):
        if cache_path:
            self.cache_path = os.path.abspath(cache_path)
        else:
            self.cache_path = os.path.join(tempfile.gettempdir(), ".diskcache")

    def __call__(self, func):
        """
        Returns a wrapped function.
        If there is no cache on disk, the function is called to get the result, cached and returned
        If there is a cache on the disk, the cached result is returned directly
        :param func:
        """

        @func_wraps(func)
        def wrapper(*args, **kw):
            """
            wrapper
            """
            params_uuid = uuid.uuid5(self._NAMESPACE, "-".join(map(str, (args, kw))))
            key = '{}-{}.cache'.format(func.__name__, str(params_uuid))
            cache_file = os.path.join(self.cache_path, key)

            if not os.path.exists(self.cache_path):
                os.makedirs(self.cache_path)

            try:
                with open(cache_file, 'rb') as f:
                    val = pickle.load(f)
            except BaseException as msg:
                log.warning(msg)
                val = func(*args, **kw)
                try:
                    with open(cache_file, 'wb') as f:
                        pickle.dump(val, f)
                except BaseException as msg:
                    log.warning(msg)
            return val

        return wrapper

    def clear(self, func_name: str = None) -> None:
        """
        clear function cache
        :param func_name:
        :return:
        """
        if func_name is None:
            log.info("💾 Clear all function cache")
            if os.path.exists(self.cache_path):
                shutil.rmtree(self.cache_path)
        else:
            log.info(f"💾 Clear function cache: {func_name}")
            for cache_file in os.listdir(self.cache_path):
                if cache_file.startswith(func_name + "-"):
                    os.remove(os.path.join(self.cache_path, cache_file))


disk_cache = DiskCache



import base64

from playwright.sync_api import Page


def screenshot_base64(page: Page):
    """
    Obtain a base64 encoded screenshot
    :param page:
    :return:
    """
    img = page.screenshot(type='png', path=None)
    img_base64 = base64.b64encode(img).decode('utf-8')
    return img_base64



"""
SQL API
"""


class SQLBase:
    """SQL base API"""

    @staticmethod
    def dict_to_str(data: dict) -> str:
        """
        dict to set str
        """
        tmp_list = []
        for key, value in data.items():
            if value is None:
                tmp = f"{key}=null"
            elif isinstance(value, int):
                tmp = f"{key}={value}"
            else:
                tmp = f"{key}='{value}'"
            tmp_list.append(tmp)
        return ','.join(tmp_list)

    @staticmethod
    def dict_to_str_and(conditions: dict) -> str:
        """
        dict to where and str
        """
        tmp_list = []
        for key, value in conditions.items():
            if value is None:
                tmp = f"{key}=null"
            elif isinstance(value, int):
                tmp = f"{key}={value}"
            else:
                tmp = f"{key}='{value}'"
            tmp_list.append(tmp)
        return ' and '.join(tmp_list)

    def delete(self, table: str, where: dict = None) -> None:
        """
        delete table data
        """
        return self.delete_data(table, where)

    def insert(self, table: str, data: dict) -> None:
        """
        insert sql statement
        """
        return self.insert_data(table, data)

    def select(self, table: str, where: dict = None, one: bool = False) -> list:
        """
        select sql statement
        """
        return self.select_data(table, where, one)

    def update(self, table: str, data: dict, where: dict) -> None:
        """
        update sql statement
        """
        return self.update_data(table, data, where)
